package nemo.model;

import java.io.*;
import java.util.*;

public class Nemo {
	private int nRows;
	private int nCols;
	protected Vector listeners = new Vector();
	private Row[] row;
	private Col[] col;
	private Vector cellListeners = new Vector();

	public Nemo(int nRows, int nCols) {
		init(nRows, nCols);
	}

	private void init(int nRows, int nCols) {
		this.nRows = nRows;
		this.nCols = nCols;

		Cell[][] cell = new Cell[nRows][nCols];
		for (int i = 0; i < nRows; i++) {
			for (int j = 0; j < nCols; j++) {
				cell[i][j] = new Cell();
			}
		}
		
		row = new Row[nRows];
		for (int i = 0; i < nRows; i++) {
			row[i] = new Row(this, i, cell, null);
		}
		
		col = new Col[nCols];
		for (int i = 0; i < nCols; i++) {
			col[i] = new Col(this, i, cell, null);
		}
	}

	public int getNRows() {
		return nRows;
	}

	public int getNCols() {
		return nCols;
	}

	public boolean marked(int row, int col) {
		return getRow(row).get(col).marked();
	}

	public void mark(int row, int col) {
		unflag(row, col);
		getRow(row).get(col).mark(true);
		sendCellEvent(new CellEvent(this, CellEvent.CELL_MODIFIED, row, col));
	}

	public boolean finished() {
		for (int i = 0; i < nRows; i++) {
			if (!row[i].finished())
				return false;
		}
		for (int i = 0; i < nCols; i++) {
			if (!col[i].finished())
				return false;
		}
		return true;
	}

	public Numbers getRowNumbers(int i) {
		return row[i].getNumbers();
	}

	public Numbers getColNumbers(int i) {
		return col[i].getNumbers();
	}

	public Nemo(Reader in) throws IOException {
		load(in);
	}

	private Numbers parseLine(BufferedReader in) throws IOException {
		String s = in.readLine();
		Numbers numbers = new Numbers();
		StringTokenizer st = new StringTokenizer(s);
		while (st.hasMoreTokens()) {
			numbers.add(Integer.parseInt(st.nextToken()));
		}
		return numbers;
	}

	public void unmark(int row, int col) {
		unflag(row, col);
		getRow(row).get(col).mark(false);
		sendCellEvent(new CellEvent(this, CellEvent.CELL_MODIFIED, row, col));
	}

	public void load(Reader in) throws IOException {
		BufferedReader br = new BufferedReader(in);
		Numbers numbers = parseLine(br);
		init(numbers.get(0), numbers.get(1));
		for (int i = 0; i < nRows; i++) {
			row[i].setNumbers(parseLine(br));
		}
		for (int i = 0; i < nCols; i++) {
			col[i].setNumbers(parseLine(br));
		}
		sendEvent();
	}

	public void save(Writer writer) {
		PrintWriter out = new PrintWriter(writer);
		out.println(nRows + " " + nCols);
		for (int i = 0; i < nRows; i++) {
			out.println(row[i].getNumbers().toString());
		}
		for (int i = 0; i < nCols; i++) {
			out.println(col[i].getNumbers().toString());
		}
	}

	public void makeNumbers() {
		for (int i = 0; i < nRows; i++) {
			row[i].setNumbers(Marks.toNumbers(getRow(i).getMarks()));
		}
		for (int i = 0; i < nCols; i++) {
			col[i].setNumbers(Marks.toNumbers(getCol(i).getMarks()));
		}
	}

	public void addNemoListener(NemoListener al) {
		listeners.addElement(al);
	}

	public void sendEvent() {
		NemoEvent ne = new NemoEvent(this);
		for (Enumeration e = listeners.elements(); e.hasMoreElements();)
			((NemoListener) e.nextElement()).nemoLoaded(ne);
	}

	public boolean checkRow(int row) {
		return Marks.toNumbers(getRow(row).getMarks()).equals(getRow(row).getNumbers());
	}

	public boolean checkCol(int col) {
		return Marks.toNumbers(getCol(col).getMarks()).equals(getCol(col).getNumbers());
	}

	public boolean flagged(int row, int col) {
		return getRow(row).get(col).flagged();
	}

	public void flag(int row, int col) {
		unmark(row, col);
		getRow(row).get(col).flag(true);
		sendCellEvent(new CellEvent(this, CellEvent.CELL_MODIFIED, row, col));
	}

	public void unflag(int row, int col) {
		getRow(row).get(col).flag(false);
		sendCellEvent(new CellEvent(this, CellEvent.CELL_MODIFIED, row, col));
	}

	public void unflagAll() {
		for (int i = 0; i < nRows; i++) {
			for (int j = 0; j < nCols; j++) {
				unflag(i, j);
			}
		}
	}
	
	public Row getRow(int row) {
		return this.row[row];
	}
	
	public Row getCol(int col) {
		return this.col[col];
	}
	
	public Row[] getRows() {
		Vector result = new Vector();
		for (int i = 0; i < nRows; i++) {
			result.addElement(getRow(i));
		}
		return (Row[]) result.toArray(new Row[0]);
	}
	
	/**
	 * Method getCols.
	 * @return Row[]
	 */
	public Row[] getCols() {
		Vector result = new Vector();
		for (int i = 0; i < nCols; i++) {
			result.addElement(getCol(i));
		}
		return (Row[]) result.toArray(new Row[0]);
	}

	public void addCellListener(CellListener l) {
		cellListeners.addElement(l);
	}
	
	private void sendCellEvent(CellEvent e) {
		for (Iterator i = cellListeners.iterator(); i.hasNext(); ) {
			CellListener l = (CellListener) i.next();
			l.cellModified(e);
		}
	}
}
