package nemo.model;
import java.util.*;

public class Permuter {
	private static int max = 100;
	
	public static BitSet[] permute(int w, int[] n, SolverRow sr) {
		Vector v = permuteHelper(w, n, 0, 0, sr);
		if (v == null) {
			return null;
		}
		
		return (BitSet[]) v.toArray(new BitSet[0]);
	}

	// |_______************_**********_..._****************|
	//  <- r -><-  n[0]  -> <- n[1] -> ... <-    n[k]    ->
	//  <-                        w                      ->
	public static Vector permuteHelper(int w, int[] n, int numSoFar, int offset, SolverRow sr) {
		//System.out.println(w + ", " + n + ", " + offset);
		final int k = n.length - 1;
		int r = w - (sum(n) + k);
		Vector result = new Vector();
		for (int i = 0; i <= r; i++) {
			if (n.length > 1) {
				// m = n[1..k]
				int[] m = new int[k];
				System.arraycopy(n, 1, m, 0, m.length);
				final int ww = w - i - n[0] - 1;
				Vector t = permuteHelper(ww, m, numSoFar, offset + i + n[0] + 1, sr);
				if (t == null) {
					return null;
				}
				for (int j = 0; j < t.size(); j++) {
					BitSet candidate = new BitSet();
					candidate.set(offset, offset + i, false);
					candidate.set(offset + i, offset + i + n[0]);
					candidate.or((BitSet) t.get(j));
					if (candidate.length() > offset + w) {
						throw new Error("permutation error");
					}
					if (offset == 0) {
						if (sr != null && !sr.isValid(candidate)) {
							continue;
						}
						//System.out.println("candidate: " + candidate);
					}
					if (++numSoFar > max) {
//						System.out.println("# permutations > " + max + 
//								" delaying...");
						return null;
					}
					result.addElement(candidate);
				}
			} else {
				if (n.length < 1)
					n = new int[] {0};
				BitSet candidate = new BitSet();
				candidate.set(offset, offset + i, false);
				candidate.set(offset + i, offset + i + n[0]);
				//candidate.set(i + n[0], w, false);
				if (offset == 0) {
					if (sr != null && !sr.isValid(candidate)) {
						continue;
					}
					//System.out.println("candidate: " + candidate);
				}
				if (++numSoFar > max) {
//					System.out.println("# permutations > " + max + 
//							" delaying...");
					return null;
				}
				result.addElement(candidate);
			}
		}
		return result;
	}

	public static String repeat(char c, int n) {
		String result = "";
		for (int i = 0; i < n; i++) {
			result += c;
		}
		return result;
	}

	public static int sum(int[] n) {
		int result = 0;
		for (int i = 0; i < n.length; i++) {
			result += n[i];
		}
		return result;
	}

	public static int getMax() {
		return max ;
	}

	public static void setMax(int i) {
		max = i;
		System.out.println("Permuter: max = " + max);
	}
}
