package nemo.model;

import java.util.BitSet;


/**
 * @author Changbeom Ahn
 */
public class SolverRow implements Comparable {
	private Row row;
	private BitSet[] p = null;
	private boolean isRow;
	private int num;
	
	/**
	 * @deprecated Use newer constructor.
	 */
	public SolverRow(Row row) {
		this.row = row;
		p = Permuter.permute(size(), row.getNumbers().intArray(), 
				this);
	}
	
	public Row getRow() {
		return row;
	}
	
	/**
	 * Method countPermutations.
	 * @return int
	 */
	public int countPermutations() {
		if (p == null) {
			return 1;
		}
		return p.length;
	}

	/**
	 * Method getPermutation.
	 * @param j
	 * @return Object
	 */
	public BitSet getPermutation(int j) {
		if (p == null) {
			generatePermutations();
			if (p == null) {
				return new BitSet();
			}
		}
		return p[j];
	}

	private void generatePermutations() {
		p = Permuter.permute(size(), row.getNumbers().intArray(), this);
		if (p == null) {
			return;
		}
		System.out.println("generated " + p.length + " permutations for " + 
				row.getNumbers());
	}

	/**
	 * Method disablePermutation.
	 * @param k
	 */
	public void disablePermutation(int k) {
		p[k] = null;
	}

	/**
	 * Method markPermutation.
	 * @param i
	 */
	public void setPermutation(int j) {
		set(p[j]);
	}
	
	public void set(BitSet set) {
		int n = isRow ? row.getNemo().getNCols() : row.getNemo().getNRows();
		for (int i = 0; i < n; i++) {
			if (set.get(i)) {
				row.getNemo().mark(isRow ? num : i, isRow ? i : num);
			} else {
				row.getNemo().unmark(isRow ? num : i, isRow ? i : num);
			}
		}
	}
	
	public BitSet[] combine() {
		if (p == null) {
			generatePermutations();
			if (p == null) {
				return null;
			}
		}

		int size = size();
		
		BitSet combinedMarks = new BitSet(size);
		combinedMarks.set(0, size);
		
		BitSet combinedFlags = new BitSet(size);
		
		for (int k = 0; k < p.length; k++) {
			if (p[k] != null) {
				combinedMarks.and(p[k]);
				combinedFlags.or(p[k]);
			}
		}
		
		combinedFlags.flip(0, size);
		
		BitSet[] result = new BitSet[2];
		result[0] = combinedMarks;
		result[1] = combinedFlags;
		return result;
	}
	
	public void mark(BitSet[] sets) {
		if (sets == null) {
			return;
		}
		for (int i = 0; i < size(); i++) {
			if (sets[0].get(i)) {
				row.getNemo().mark(isRow ? num : i, isRow ? i : num);
			} else if (sets[1].get(i)) {
				row.getNemo().flag(isRow ? num : i, isRow ? i : num);
			}
		}
	}
	
	public void checkEmpty() {
		Numbers zero = new Numbers();
		zero.add(0);
		if (row.getNumbers().size() == 0 || row.getNumbers().equals(zero)) {
			for (int i = 0; i < size(); i++) {
				row.getNemo().flag(isRow ? num : i, isRow ? i : num);
			}
		}
	}
	
	public int size() {
		return isRow ? row.getNemo().getNCols() : row.getNemo().getNRows();
	}
	
	/**
	 * Method checkFull.
	 */
	public void checkFull() {
		Numbers numbers = row.getNumbers();
		if (availableSize() == numbers.sum() + numbers.size() - 1) {
			int c = countLeadingFlags();
			for (int j = 0; j < numbers.get(0); j++) {
				row.getNemo().mark(isRow ? num : c, isRow ? c : num);
				c++;
			}
			for (int i = 1; i < numbers.size(); i++) {
				row.getNemo().flag(isRow ? num : c, isRow ? c : num);
				c++;
				for (int j = 0; j < numbers.get(i); j++) {
					row.getNemo().mark(isRow ? num : c, isRow ? c : num);
					c++;
				}
			}
		}
	}
	
	private int availableSize() {
		return size() - countLeadingFlags() - countTrailingFlags();
	}
	
	private int countLeadingFlags() {
		int i;
		for (i = 0; i < size(); i++) {
			if (!row.get(i).flagged()) {
				break;
			}
		}
		return i;
	}

	private int countTrailingFlags() {
		int i;
		for (i = size() - 1; i >= 0; i--) {
			if (!row.get(i).flagged()) {
				break;
			}
		}
		return size() - 1 - i;
	}

	/**
	 * Method checkPermutations.
	 */
	public void checkPermutations() {
		if (p == null) {
			generatePermutations();
			if (p == null) {
				return;
			}
		}

		for (int j = 0; j < p.length; j++) {
			if (p[j] == null) {
				continue;
			}
			if (!isValid(p[j])) {
				p[j] = null;
			}
		}
	}

	public SolverRow(Row row, boolean isRow, int num) {
		this.row = row;
		this.isRow = isRow;
		this.num = num;
	}
	
	public int getPoint() {
		int result = 0;
		Numbers numbers = row.getNumbers();
		for (int i = 0; i < numbers.size(); ++i) {
			result += numbers.get(i) * numbers.get(i);
		}
		if (result == 0) {
			result = 10000;
		}
		
		int multiplier = 1;
		if (numbers.size() < 10) {
			multiplier = 10 - numbers.size();
		}
		
		return result * multiplier;
	}

	public int compareTo(Object o) {
		return - (getPoint() - ((SolverRow) o).getPoint());
	}

	public boolean isValid(BitSet candidate) {
		BitSet marks = row.getMarks();
		BitSet clone = (BitSet) candidate.clone();
		clone.and(marks);
		if (!clone.equals(marks)) {
			return false;
		}
		
		BitSet flags = row.getFlags();
		clone = (BitSet) candidate.clone();
		clone.and(flags);
		if (clone.intersects(flags)) {
			return false;
		}

		return true;
	}

	public void checkFinished() {
		if (row.finished()) {
			for (int i = 0; i < size(); ++i) {
				Cell cell = row.get(i);
				if (!cell.marked() && !cell.flagged()) {
					cell.flag(true);
				}
			}
		}
	}
}
