package nemo.swingui;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import nemo.model.CellEvent;
import nemo.model.CellListener;
import nemo.model.Nemo;
import nemo.model.NemoEvent;
import nemo.model.NemoListener;

public class Picture extends JComponent implements NemoListener, CellListener {
	private Nemo nemo;
	private Dimension d;
	private int height;

	public Picture(Nemo nemo) {
		this.nemo = nemo;
		addMouseListener();
		addMouseMotionListener();
		nemo.addNemoListener(this);
		nemo.addCellListener(this);
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		// g.drawRect(0, 0, d.width-1, d.height-1);
		Color save = g.getColor();
		for (int row = 0; row < nemo.getNRows(); row++) {
			for (int col = 0; col < nemo.getNCols(); col++) {
				if (nemo.marked(row, col)) {
					g.setColor(Color.darkGray);
				} else {
					g.setColor(Color.lightGray);
				}
				g.fillRect(1 + col*height, 1 + row*height,
					height - 2, height - 2);
				if (nemo.flagged(row, col)) {
					int x = col*height + 1;
					int y = row*height + 1;
					g.setColor(Color.red);
					g.drawLine(x, y, x+height-2, y+height-2);
					g.drawLine(x, y+height-2, x+height-2, y);
				}
			}
		}
		g.setColor(save);
	}

	public Dimension getPreferredSize() {
		return getMinimumSize();
	}
	
	public Dimension getMinimumSize() {
		FontMetrics fm = getGraphics().getFontMetrics();
		height = fm.getHeight();
		d = new Dimension(nemo.getNCols() * height + 1,
			nemo.getNRows() * height + 1);
		return d;
	}

	private void addMouseListener() {
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				handleMouseEvent(e);
			}
		});
	}

	private void addMouseMotionListener() {
		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {
				handleMouseEvent(e);
			}
		});
	}

	private void handleMouseEvent(MouseEvent e) {
		if (nemo.finished())
			return;
		int row = e.getY() / height;
		int col = e.getX() / height;
		if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) {
			nemo.mark(row, col);
		} else if ((e.getModifiers() & MouseEvent.BUTTON2_MASK) != 0) {
			nemo.flag(row, col);
		} else {
			nemo.unmark(row, col);
		}
		repaint();
		if (nemo.finished()) {
			// System.out.println("finished!");
			JOptionPane.showMessageDialog(null,
				"Finished!");
			// System.exit(0);
		}
	}
	
	public void nemoLoaded(NemoEvent e) {
		repaint();
	}

	/* (non-Javadoc)
	 * @see nemo.model.CellListener#cellModified(nemo.model.CellEvent)
	 */
	public void cellModified(CellEvent e) {
		repaint();
	}
}
