package nemo.tests;
import junit.framework.TestCase;
import java.io.*;
import nemo.model.Nemo;

public class NemoTest extends TestCase {
	private Nemo nemo;
	private static final int N = 3;
	private StringReader in;
	private String s;

	public NemoTest(String name) {
		super(name);
	}

	public void testMarked() {
		assertTrue(!nemo.marked(0, 0));
	}

	public void testMark() {
		nemo.mark(0, 0);
		assertTrue(nemo.marked(0, 0));
	}

	protected void setUp() {
		nemo = new Nemo(N, N);
		String ls = System.getProperty("line.separator");
		s = "3 3" + ls + "3" + ls + "1" + ls + "1" + ls + "1 1" + ls
			+ "2" + ls + "1" + ls;
		in = new StringReader(s);
	}

	public void testFinished() {
		try {
			nemo = new Nemo(new StringReader(s));
		}
		catch (IOException e) {
			fail();
		}
		assertTrue(!nemo.finished());
		for (int i = 0; i < 3; i++) {
			nemo.mark(0, i);
			nemo.mark(3 - i - 1, i);
		}
		assertTrue(nemo.finished());
	}

	public void testUnmark() {
		nemo.mark(0, 0);
		assertTrue(nemo.marked(0, 0));
		nemo.unmark(0, 0);
		assertTrue(!nemo.marked(0, 0));
	}

	public void testLoad() {
		try {
			nemo.load(in);
		}
		catch (IOException e) {
			fail();
		}
		assertEquals(N, nemo.getNRows());
		assertEquals(N, nemo.getNCols());
		assertEquals("3", nemo.getRowNumbers(0).toString());
		assertEquals("1", nemo.getRowNumbers(1).toString());
		assertEquals("1", nemo.getRowNumbers(2).toString());
		assertEquals("1 1", nemo.getColNumbers(0).toString());
		assertEquals("2", nemo.getColNumbers(1).toString());
		assertEquals("1", nemo.getColNumbers(2).toString());
	}

	public void testSave() throws IOException {
		nemo.load(in);
		StringWriter out = new StringWriter();
		nemo.save(out);
		assertEquals(s, out.toString());
	}

	public void testMakeNumbers() {
		for (int i = 0; i < N; i++) {
			nemo.mark(i, i);
		}
		nemo.makeNumbers();
		for (int i = 0; i < N; i++) {
			assertEquals("1", nemo.getRowNumbers(i).toString());
			assertEquals("1", nemo.getColNumbers(i).toString());
		}
	}

	public void testFlagged() {
		for (int i = 0; i < nemo.getNRows(); i++) {
			for (int j = 0; j < nemo.getNCols(); j++) {
				assertTrue(!nemo.flagged(i, j));
			}
		}
	}

	public void testFlag() {
		for (int i = 0; i < nemo.getNRows(); i++) {
			for (int j = 0; j < nemo.getNCols(); j++) {
				nemo.flag(i, j);
				assertTrue(nemo.flagged(i, j));
			}
		}
	}

	public void testUnflag() {
		for (int i = 0; i < nemo.getNRows(); i++) {
			for (int j = 0; j < nemo.getNCols(); j++) {
				nemo.flag(i, j);
				assertTrue(nemo.flagged(i, j));
				nemo.unflag(i, j);
				assertTrue(!nemo.flagged(i, j));
			}
		}
	}
}
